@@ -26,6 +26,7 @@
 /*----------------------------------------------------------------------------*/
 
 #include <libecal/e-cal.h>
+#include <libedata-cal/e-data-cal-types.h>
 
 #include <libekolabutil/camel-system-headers.h>
 #include <libekolab/kolab-settings-handler.h>
@@ -46,8 +47,8 @@ kolab_util_calendar_create_http_request (KolabSettingsHandler *ksettings,
 	const gchar *user = NULL;
 	const gchar *passwd = NULL;
 	guint tls_variant = KOLAB_TLS_VARIANT_DEFAULT;
-	GError *tmp_error = NULL;
 	KolabUtilHttpJob *job = NULL;
+	GError *tmp_err = NULL;
 
 	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
 	g_assert (path != NULL);
@@ -55,21 +56,21 @@ kolab_util_calendar_create_http_request (KolabSettingsHandler *ksettings,
 
 	url = kolab_settings_handler_get_char_field (ksettings,
 	                                             KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_URI,
-	                                             &tmp_error);
+	                                             &tmp_err);
 	if (url == NULL) {
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		return NULL;
 	}
 	tls_variant = kolab_settings_handler_get_uint_field (ksettings,
 	                                                     KOLAB_SETTINGS_HANDLER_UINT_FIELD_TLS_VARIANT,
-	                                                     &tmp_error);
-	if (error != NULL) {
-		g_propagate_error (error, tmp_error);
+	                                                     &tmp_err);
+	if (tmp_err != NULL) {
+		g_propagate_error (error, tmp_err);
 		return NULL;
 	}
-	uri = camel_url_new (url, &tmp_error);
-	if (tmp_error != NULL) {
-		g_propagate_error (error, tmp_error);
+	uri = camel_url_new (url, &tmp_err);
+	if (tmp_err != NULL) {
+		g_propagate_error (error, tmp_err);
 		return NULL;
 	}
 	if (uri == NULL) {
@@ -86,29 +87,25 @@ kolab_util_calendar_create_http_request (KolabSettingsHandler *ksettings,
 	camel_url_set_path (uri, path);
 	user = kolab_settings_handler_get_char_field (ksettings,
 	                                              KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_NAME,
-	                                              &tmp_error);
+	                                              &tmp_err);
 	if (user == NULL) {
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		camel_url_free (uri);
 		return NULL;
 	}
 	camel_url_set_user (uri, user);
 	passwd = kolab_settings_handler_get_char_field (ksettings,
 	                                                KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_PASSWORD,
-	                                                &tmp_error);
+	                                                &tmp_err);
 	if (passwd == NULL) {
-		g_propagate_error (error, tmp_error);
-		camel_url_free (uri);
-		return NULL;
+		g_warning ("%s()[%u]: no password supplied! (%s)",
+		           __func__, __LINE__, tmp_err->message);
+		g_error_free (tmp_err);
 	}
 
-	/* FIXME CamelURL does no longer carry auth details */
-	g_error ("%s: FIXME CamelURL does no longer carry auth details",
-	         __func__);
-	/* camel_url_set_passwd (uri, passwd); */
-
 	job = kolab_util_http_job_new ();
 	job->url = uri;
+	job->passwd = g_strdup (passwd); /* no longer carried by CamelURL */
 
 	return job;
 } /* kolab_util_calendar_create_http_request () */
@@ -122,7 +119,7 @@ kolab_util_calendar_create_pfb_trigger (KolabSettingsHandler *ksettings,
 	gchar *path = NULL;
 	const gchar *username = NULL;
 	gint source_offset = 0;
-	GError *tmp_error = NULL;
+	GError *tmp_err = NULL;
 
 	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
 	g_assert (sourcename != NULL);
@@ -130,9 +127,9 @@ kolab_util_calendar_create_pfb_trigger (KolabSettingsHandler *ksettings,
 
 	username = kolab_settings_handler_get_char_field (ksettings,
 	                                                  KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_NAME,
-	                                                  &tmp_error);
-	if (tmp_error != NULL) {
-		g_propagate_error (error, tmp_error);
+	                                                  &tmp_err);
+	if (tmp_err != NULL) {
+		g_propagate_error (error, tmp_err);
 		return NULL;
 	}
 
@@ -143,10 +140,10 @@ kolab_util_calendar_create_pfb_trigger (KolabSettingsHandler *ksettings,
 
 	job = kolab_util_calendar_create_http_request (ksettings,
 	                                               path,
-	                                               &tmp_error);
+	                                               &tmp_err);
 	g_free (path);
 	if (job == NULL) {
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		return NULL;
 	}
 	return job;
@@ -159,7 +156,7 @@ kolab_util_calendar_create_xfb_request (KolabSettingsHandler *ksettings,
 {
 	KolabUtilHttpJob *job = NULL;
 	gchar *path = NULL;
-	GError *tmp_error = NULL;
+	GError *tmp_err = NULL;
 
 	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
 	g_assert (query != NULL);
@@ -169,10 +166,10 @@ kolab_util_calendar_create_xfb_request (KolabSettingsHandler *ksettings,
 
 	job = kolab_util_calendar_create_http_request (ksettings,
 	                                               path,
-	                                               &tmp_error);
+	                                               &tmp_err);
 	g_free (path);
 	if (job == NULL) {
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		g_warning ("%s()[%u] call to kolab_util_calendar_create_http_request() returned NULL!",
 		           __func__, __LINE__);
 		return NULL;
@@ -241,7 +238,7 @@ kolab_util_calendar_toggle_pfb_trigger (KolabSettingsHandler *ksettings,
 	 * If an error occured, internal_state is set to false as indicator.
 	 */
 	KolabUtilHttpJob *job = NULL;
-	GError *tmp_error = NULL;
+	GError *tmp_err = NULL;
 
 	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
 	g_assert (sourcename != NULL);
@@ -249,19 +246,19 @@ kolab_util_calendar_toggle_pfb_trigger (KolabSettingsHandler *ksettings,
 
 	job = kolab_util_calendar_create_pfb_trigger (ksettings,
 	                                              sourcename,
-	                                              &tmp_error);
+	                                              &tmp_err);
 	if (job == NULL) {
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		g_warning ("%s()[%u] error: could not create F/B toggle, giving up on %s.",
 		           __func__, __LINE__, sourcename);
 		return FALSE;
 	}
 	job->buffer = g_byte_array_new ();
-	(void)kolab_util_http_get (job, &tmp_error);
+	(void)kolab_util_http_get (job, &tmp_err);
 	kolab_util_http_job_free (job);
 
-	if (tmp_error != NULL) {
-		g_propagate_error (error, tmp_error);
+	if (tmp_err != NULL) {
+		g_propagate_error (error, tmp_err);
 		return FALSE;
 	}
 	return TRUE;
@@ -273,7 +270,7 @@ kolab_util_calendar_retrieve_xfb (KolabSettingsHandler *ksettings,
                                   GError **error)
 {
 	KolabUtilHttpJob *job = NULL;
-	GError *tmp_error = NULL;
+	GError *tmp_err = NULL;
 
 	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
 	g_assert (query != NULL);
@@ -281,20 +278,20 @@ kolab_util_calendar_retrieve_xfb (KolabSettingsHandler *ksettings,
 
 	job = kolab_util_calendar_create_xfb_request (ksettings,
 	                                              (gchar *) query,
-	                                              &tmp_error);
+	                                              &tmp_err);
 	if (job == NULL) {
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		g_warning ("%s()[%u] error creating job, giving up on %s.",
 		           __func__, __LINE__, (gchar *) query);
 		return NULL;
 	}
 
 	job->buffer = g_byte_array_new ();
-	(void)kolab_util_http_get (job, &tmp_error);
-	if (tmp_error != NULL) {
+	(void)kolab_util_http_get (job, &tmp_err);
+	if (tmp_err != NULL) {
 		/* might happen, perhaps the server did not now about the user. */
 		kolab_util_http_job_free (job);
-		g_propagate_error (error, tmp_error);
+		g_propagate_error (error, tmp_err);
 		return NULL;
 	}
 
@@ -316,11 +313,11 @@ kolab_util_calendar_store (ECalComponent *ecalcomp,
 	KolabMailAccessOpmodeID mode = KOLAB_MAIL_ACCESS_OPMODE_INVAL;
 	gboolean do_trigger = FALSE;
 	gboolean ok = FALSE;
-	GError *tmp_error = NULL;
+	GError *tmp_err = NULL;
 
 	g_assert (E_IS_CAL_COMPONENT (ecalcomp));
 	/* ecaltz may be NULL */
-	g_assert (E_IS_CAL_COMPONENT (default_tz));
+	(void)default_tz; /* FIXME needed? */ /* default_tz may be NULL */
 	g_assert (KOLAB_IS_MAIL_ACCESS (koma));
 	g_assert (uri != NULL);
 	/* cancellable may be NULL */
@@ -341,13 +338,13 @@ kolab_util_calendar_store (ECalComponent *ecalcomp,
 	                                      kmh,
 	                                      sourcename,
 	                                      cancellable,
-	                                      &tmp_error);
+	                                      &tmp_err);
 	if (! ok)
 		goto cleanup;
 
 	mode = kolab_mail_access_get_opmode (koma,
-	                                     &tmp_error);
-	if (tmp_error != NULL)
+	                                     &tmp_err);
+	if (tmp_err != NULL)
 		goto cleanup;
 	if (mode < KOLAB_MAIL_ACCESS_OPMODE_ONLINE) {
 		/* In OFFLINE state, the triggers are generated during
@@ -358,19 +355,31 @@ kolab_util_calendar_store (ECalComponent *ecalcomp,
 
 	do_trigger = kolab_mail_access_source_fbtrigger_needed (koma,
 	                                                        sourcename,
-	                                                        &tmp_error);
-	if (tmp_error != NULL)
+	                                                        &tmp_err);
+	if (tmp_err != NULL)
 		goto cleanup;
 
-	if (do_trigger == TRUE)
+	if (do_trigger == TRUE) {
 		(void) kolab_util_calendar_toggle_pfb_trigger (ksettings,
 		                                               sourcename,
-		                                               &tmp_error);
+		                                               &tmp_err);
+		if (tmp_err != NULL) {
+			/* if triggering the XFB generation fails,
+			 * this is not a critical failure regarding
+			 * storing the PIM object
+			 */
+			g_warning ("%s()[%u] %s",
+			           __func__, __LINE__, tmp_err->message);
+			g_error_free (tmp_err);
+			tmp_err = NULL;
+		}
+	}
+
  cleanup:
 	g_object_unref (ksettings);
 	g_free (sourcename);
-	if (tmp_error != NULL) {
-		g_propagate_error (error, tmp_error);
+	if (tmp_err != NULL) {
+		g_propagate_error (error, tmp_err);
 		return FALSE;
 	}
 	return TRUE;
@@ -410,7 +419,6 @@ kolab_util_calendar_map_error (GError *error)
 }
 #endif
 
-#if 0 /* FIXME old */
 /**
  * kolab_util_calendar_extract:
  * @icalcomp: An icalcomponent.
@@ -423,17 +431,28 @@ kolab_util_calendar_map_error (GError *error)
  *
  * Returns: if the given component type could be extracted successfully into ecalcomp + tzcomp;
  */
-ECalBackendSyncStatus
+gboolean
 kolab_util_calendar_extract (icalcomponent *icalcomp,
                              ECalSourceType source_type,
                              ECalComponent **ecalcomp,
-                             ECalComponent **tzcomp)
+                             ECalComponent **tzcomp,
+                             GError **error)
 {
 	ECalComponent *comp = NULL;
 	ECalComponent *tz = NULL;
 	icalcomponent *icalsub = NULL;
 	icalcomponent *icaltz = NULL;
 	icalcomponent_kind icalkind;
+	GError *tmp_err = NULL;
+
+	/* Caution: Let this function return errors
+	 *          only which are well-known EDataCal
+	 *          errors, since the error set here
+	 *          is propagated directly (no mapping)
+	 *          in the ECalBackendKolab
+	 */
+
+	/* TODO Function argument assertions */
 
 	switch (source_type) {
 	case E_CAL_SOURCE_TYPE_EVENT:
@@ -446,14 +465,16 @@ kolab_util_calendar_extract (icalcomponent *icalcomp,
 		icalkind = ICAL_VJOURNAL_COMPONENT;
 		break;
 	default:
-		g_error ("%s()[%u] Got an unknown value for ECalSourceType, this should not happen.",
-		         __func__, __LINE__);
+		/* can't happen */
+		g_assert_not_reached ();
 	}
 
 	icalsub = icalcomponent_get_first_component (icalcomp,
 	                                             icalkind);
 	if (icalsub == NULL) {
-		return GNOME_Evolution_Calendar_InvalidObject;
+		tmp_err = e_data_cal_create_error (InvalidObject, NULL);
+		g_propagate_error (error, tmp_err);
+		return FALSE;
 	}
 
 	comp = e_cal_component_new ();
@@ -478,6 +499,70 @@ kolab_util_calendar_extract (icalcomponent *icalcomp,
 		g_object_unref (tz);
 	}
 
-	return GNOME_Evolution_Calendar_Success;
+	return TRUE;
 } /* kolab_util_calendar_extract () */
-#endif
+
+void
+kolab_util_calendar_err_to_edb_err (GError **e_err,
+                                    const GError *k_err,
+                                    const gchar *func,
+                                    guint line)
+{
+	EDataCalCallStatus status = OtherError;
+	GError *tmp_err = NULL;
+
+	g_return_if_fail (e_err == NULL || *e_err == NULL);
+	g_return_if_fail (k_err != NULL);
+
+	g_warning ("%s()[%u]: '%s', Code %i, Domain '%s'",
+	           func, line, k_err->message, k_err->code,
+	           g_quark_to_string (k_err->domain));
+
+	if (!e_err)
+		return;
+
+	if (g_error_matches (k_err, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_propagate_error (e_err, g_error_copy (k_err));
+		return;
+	}
+
+	/* TODO
+	 *
+	 * Need to implement a more elaborated error mapping here.
+	 * This function maps Kolab engine errors only. Errors can
+	 * originate from the camel/ as well as from the libekolab/
+	 * libs. These yield errors in various domains. The following
+	 * EDataCalCallStatus (other than OTHER_ERROR) are candidates
+	 * for being populated from certain k_err's:
+	 *
+	 * RepositoryOffline
+	 * PermissionDenied
+	 * InvalidRange
+	 * ObjectNotFound
+	 * InvalidObject
+	 * ObjectIdAlreadyExists
+	 * AuthenticationFailed
+	 * AuthenticationRequired
+	 * UnsupportedField
+	 * UnsupportedMethod
+	 * TLSNotAvailable
+	 * NoSuchCal
+	 * UnknownUser
+	 * SearchSizeLimitExceeded
+	 * InvalidQuery
+	 * QueryRefused
+	 * CouldNotCancel
+	 * InvalidServerVersion
+	 * NotSupported
+	 * NotOpened
+	 *
+	 * Depending on k_err->domain and k_err->code, we can map
+	 * to a fitting EDataCalCallStatus. More than one combination
+	 * of k_err->domain and k_err->code may map to one specific
+	 * EDataCalCallStatus.
+	 */
+
+	tmp_err = e_data_cal_create_error (status,
+	                                   k_err->message);
+	g_propagate_error (e_err, tmp_err);
+}
