@@ -169,7 +169,7 @@ extd_store_get_folder_offline (CamelStore *self,
 		storage_path = g_build_filename (user_cache_dir, "folders", NULL);
 		folder_dir = imapx_path_to_physical (storage_path, folder_name);
 		g_free (storage_path);
-		/* FIXME (create CamelIMAPXExtdFolder) */
+
 		new_folder = CAMEL_FOLDER (camel_imapx_extd_folder_new (myself,
 		                                                        folder_dir,
 		                                                        folder_name,
@@ -208,19 +208,14 @@ imapx_extd_store_authenticate_sync (CamelService *service,
 	istore = CAMEL_IMAPX_STORE (service);
 
 	/* CamelIMAPXConnManager sets this before calling
-	 * camel_imapx_server_connect()(), and then clears it
+	 * camel_imapx_server_connect(), and then clears it
 	 * immediately after, all while holding the recursive
 	 * connection lock (CAMEL_SERVICE_REC_CONNECT_LOCK).
 	 * Otherwise we'd have no way of knowing which server
 	 * is trying to authenticate. */
 	server = istore->authenticating_server;
 
-	/* FIXME */
-	g_warning ("%s: FIXME getting CamelIMAPXServer, expected CamelIMAPXExtdServer",
-	           __func__);
-#if 0
 	g_return_val_if_fail (CAMEL_IS_IMAPX_EXTD_SERVER (server), CAMEL_AUTHENTICATION_REJECTED);
-#endif
 
 	/* modified */
 	return camel_imapx_extd_server_authenticate (server,
@@ -229,6 +224,31 @@ imapx_extd_store_authenticate_sync (CamelService *service,
 	                                             err);
 }
 
+static gboolean
+imapx_extd_store_connect_sync (CamelService *service,
+                               GCancellable *cancellable,
+                               GError **err)
+{
+	CamelIMAPXStore *istore = NULL;
+	CamelIMAPXServer *server = NULL;
+
+	g_assert (CAMEL_IS_SERVICE (service));
+	/* cancellable may be NULL */
+	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+
+	istore = CAMEL_IMAPX_STORE (service);
+	server = camel_imapx_extd_store_get_server (istore,
+	                                            NULL,
+	                                            cancellable,
+	                                            err);
+	if (server != NULL) {
+		g_object_unref (server);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static GList*
 imapx_extd_store_query_auth_types_sync (CamelService *service,
                                         GCancellable *cancellable,
@@ -331,13 +351,9 @@ imapx_extd_store_get_folder_sync (CamelStore *self,
 	                                        foldername,
 	                                        flags,
 	                                        err);
-	/* FIXME */
-	g_warning ("%s: FIXME getting CamelIMAPXFolder, expected CamelIMAPXExtdFolder",
-	           __func__);
-#if 0
 	if (folder != NULL)
 		g_assert (CAMEL_IS_IMAPX_EXTD_FOLDER (folder));
-#endif
+
 	return folder;
 }
 
@@ -456,6 +472,9 @@ imapx_extd_store_delete_folder_sync (CamelStore *self,
 	                                             foldername,
 	                                             cancellable,
 	                                             err);
+
+	/* FIXME delete metadata from CamelIMAPXExtdServer */
+
 	return ok;
 }
 
@@ -479,6 +498,9 @@ imapx_extd_store_rename_folder_sync (CamelStore *self,
 	                                             newname,
 	                                             cancellable,
 	                                             err);
+
+	/* FIXME update metadata in CamelIMAPXExtdServer */
+
 	return ok;
 }
 
@@ -532,10 +554,15 @@ imapx_extd_store_get_server (CamelIMAPXStore *self,
 	/* cancellable may be NULL */
 	g_return_val_if_fail (err == NULL || *err == NULL, NULL);
 
-	server = camel_imapx_store_get_server (self,
-	                                       foldername,
-	                                       cancellable,
-	                                       &tmp_err);
+	camel_service_lock (CAMEL_SERVICE (self), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	server = camel_imapx_conn_manager_get_connection (self->con_man,
+	                                                  foldername,
+	                                                  cancellable,
+	                                                  &tmp_err);
+
+	camel_service_unlock (CAMEL_SERVICE (self), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
 	if (server == NULL) {
 		g_propagate_error (err, tmp_err);
 		return NULL;
@@ -717,7 +744,7 @@ camel_imapx_extd_store_class_init (CamelIMAPXExtdStoreClass *klass)
 
 	service_class->settings_type = CAMEL_TYPE_IMAPX_SETTINGS; /* maybe need to get our own here */
 	service_class->get_name = parent_service_class->get_name;
-	service_class->connect_sync = parent_service_class->connect_sync;
+	service_class->connect_sync = imapx_extd_store_connect_sync;
 	service_class->disconnect_sync = parent_service_class->disconnect_sync;
 	service_class->authenticate_sync = imapx_extd_store_authenticate_sync;
 	service_class->query_auth_types_sync = imapx_extd_store_query_auth_types_sync;
