@@ -39,7 +39,15 @@
 #define KOLAB_CAMEL_PROVIDER_NAME       "Kolab2"
 #define KOLAB_CAMEL_URI_PREFIX          "kolab2://"
 
-#define KOLAB_CAMELURL_DUMMY_URL "protocol://user@host.domain.tld/"
+#define KOLAB_CAMELURL_DUMMY_URL           "protocol://user@host.domain.tld/"
+#define KOLAB_CAMELURL_PARAM_SSL           "use_ssl"
+#define KOLAB_CAMELURL_TLSVARIANT_NONE     "never"
+#define KOLAB_CAMELURL_TLSVARIANT_SSL      "always"
+#define KOLAB_CAMELURL_TLSVARIANT_STARTTLS "when-possible"
+
+#define KOLAB_IMAP_CLIENT_X_EVO_UID_HEADER "X-Evolution-MimeMessage-IMAP-UID"
+#define KOLAB_IMAP_CLIENT_DUMMY_FROM_NAME  "Nobody"
+#define KOLAB_IMAP_CLIENT_DUMMY_FROM_ADDR  "nobody@localhost.localdomain"
 
 /*----------------------------------------------------------------------------*/
 /* Camel init/shutdown */
